package org.samcrow;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Finds routes<br />
 * Based mostly on <a href="http://www.vogella.de/articles/JavaAlgorithmsDijkstra/article.html">http://www.vogella.de/articles/JavaAlgorithmsDijkstra/article.html</a>
 * @author Sam Crow
 *
 */
public class DijkstraAlgorithm {

	private List<Link> Links;
	private Set<Node> settledNodes;
	private Set<Node> unSettledNodes;
	private Map<Node, Node> predecessors;
	private Map<Node, Double> distance;
	public DijkstraAlgorithm(Graph graph) {
		this.Links = new ArrayList<Link>(graph.links);
	}

	public void execute(Node source) {
		settledNodes = new HashSet<Node>();
		unSettledNodes = new HashSet<Node>();
		distance = new HashMap<Node, Double>();
		predecessors = new HashMap<Node, Node>();
		distance.put(source, (double) 0);
		unSettledNodes.add(source);
		while (unSettledNodes.size() > 0) {
			Node node = getMinimum(unSettledNodes);
			settledNodes.add(node);
			unSettledNodes.remove(node);
			findMinimalDistances(node);
		}
	}

	private void findMinimalDistances(Node node) {
		List<Node> adjacentNodes = getNeighbors(node);
		for (Node target : adjacentNodes) {
			if (getShortestDistance(target) > getShortestDistance(node)
					+ getDistance(node, target)) {
				distance.put(target, getShortestDistance(node)
						+ getDistance(node, target));
				predecessors.put(target, node);
				unSettledNodes.add(target);
			}
		}

	}

	private double getDistance(Node node, Node target) {
		for (Link link : Links) {
			if ((link.startNode().equals(node) && link.endNode().equals(target))
				|| link.endNode().equals(node) && link.startNode().equals(target)
				) {
				return link.time();
			}
		}
		throw new RuntimeException("Should not happen");
	}

	private List<Node> getNeighbors(Node node) {
		List<Node> neighbors = new ArrayList<Node>();
		for (Link link : Links) {
			Node remoteNode = null;
			if (link.startNode().equals(node)
					&& !isSettled(link.endNode())) {
				remoteNode = link.endNode();
			}else if(link.endNode().equals(node) && !isSettled(link.startNode())){
				remoteNode = link.startNode();
			}
			if(remoteNode != null){
				neighbors.add(remoteNode);
			}
		}
		return neighbors;
	}

	private Node getMinimum(Set<Node> Nodees) {
		Node minimum = null;
		for (Node Node : Nodees) {
			if (minimum == null) {
				minimum = Node;
			} else {
				if (getShortestDistance(Node) < getShortestDistance(minimum)) {
					minimum = Node;
				}
			}
		}
		return minimum;
	}

	private boolean isSettled(Node Node) {
		return settledNodes.contains(Node);
	}

	private Double getShortestDistance(Node destination) {
		Double d = distance.get(destination);
		if (d == null) {
			return Double.MAX_VALUE;
		} else {
			return d;
		}
	}

	/**
	 * This method returns the path from the source to the selected target and
	 * NULL if no path exists
	 */
	public LinkedList<Node> getPath(Node target) {
		LinkedList<Node> path = new LinkedList<Node>();
		Node step = target;
		// Check if a path exists
		if (predecessors.get(step) == null) {
			return null;
		}
		path.add(step);
		while (predecessors.get(step) != null) {
			step = predecessors.get(step);
			path.add(step);
		}
		// Put it into the correct order
		Collections.reverse(path);
		return path;
	}

	
}