package org.samcrow;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;

import org.lekan.graphics.GraphicsProgram;

/**
 * Manages the display options for the map
 * @author Sam Crow
 *
 */
public class DisplayOptions implements ItemListener {
	
	public static final String STATE_CHANGE_ACTION = "DisplayOptions State Change";

	//labels and checkboxes
	private JLabel displayLabel;
	private JCheckBox displayWaypointsBox;
	private JCheckBox displayDestinationsBox;
	private JCheckBox displayDestinationLabelsBox;
	private JCheckBox displayLinksBox;
	private JCheckBox displayLinkSpeedsBox;
	
	private boolean[] defaultState = {
			 true//waypoints
			,true//destinations
			,false//destination labels
			,true//links
			,false//link speeds
	};
	
	private DisplayState state;
	
	private DisplayChangeListener listener;
	
	/**
	 * Initialize the set of options. This will add the user interface elements and set them to their default states.<br />
	 * If <code>program</code> implements DisplayChangeListener, it will automatically be added as a listener
	 * @param program the GraphicsProgram to add the elements to
	 * @param borderPosition NORTH, SOUTH, EAST, or WEST panel to add the elements to
	 */
	public DisplayOptions(GraphicsProgram program, String borderPosition){
		displayLabel = new JLabel("Display:");
		displayWaypointsBox = new JCheckBox("Waypoints", defaultState[0]);
		displayDestinationsBox = new JCheckBox("Destinations", defaultState[1]);
		displayDestinationLabelsBox = new JCheckBox("Destination Labels", defaultState[2]);
		displayLinksBox = new JCheckBox("Links", defaultState[3]);
		displayLinkSpeedsBox = new JCheckBox("Link Speeds", defaultState[4]);
		
		displayWaypointsBox.addItemListener(this);
		displayDestinationsBox.addItemListener(this);
		displayDestinationLabelsBox.addItemListener(this);
		displayLinksBox.addItemListener(this);
		displayLinkSpeedsBox.addItemListener(this);
		
		program.addJComponent(displayLabel, borderPosition);
		program.addJComponent(displayWaypointsBox, borderPosition);
		program.addJComponent(displayDestinationsBox, borderPosition);
		program.addJComponent(displayDestinationLabelsBox, borderPosition);
		program.addJComponent(displayLinksBox, borderPosition);
		program.addJComponent(displayLinkSpeedsBox, borderPosition);
		
		state = new DisplayState();
		state.displayWaypoints = defaultState[0];
		state.displayDestinations = defaultState[1];
		state.displayDestinationLabels = defaultState[2];
		state.displayLinks = defaultState[3];
		state.displayLinkSpeeds = defaultState[4];
		
		//if the GraphicsProgram implements DisplayChangeListener, automatically make it the listener
		if(program instanceof DisplayChangeListener){
			addDisplayChangeListener((DisplayChangeListener)program);
		}
	}
	
	/**
	 * Add a DisplayChangeListener to be notified when the display state changes.<br />
	 * It is not necessary to call this method if the <code>GraphicsProgram</code> passed into the constructor
	 * implements DisplayChangeListener, because it will already have been made the listener.<br />
	 * This class only supports one DisplayChangeListener. Adding one will replace the old one.
	 * @param l the listener
	 */
	public void addDisplayChangeListener(DisplayChangeListener l){
		listener = l;
	}
	
	/**
	 * Contains the state of display of various components.<br />
	 * <ul>
	 * <li>Waypoints: nodes that are waypoints, not destinations.</li>
	 * <li>Destinations: nodes that are destinations.</li>
	 * <li>Destination labels: labels for destinations.</li>
	 * <li>Links: links between nodes.</li>
	 * <li>Link speeds: labels indicating the speeds of links</li>
	 * </ul>
	 * @author Sam Crow
	 *
	 */
	public class DisplayState {
		public boolean displayWaypoints;
		public boolean displayDestinations;
		public boolean displayDestinationLabels;
		public boolean displayLinks;
		public boolean displayLinkSpeeds;
		
		/**
		 * Constructor with no arguments. This will set all data to the default values.
		 */
		public DisplayState(){
			displayWaypoints = defaultState[0];
			displayDestinations = defaultState[1];
			displayDestinationLabels = defaultState[2];
			displayLinks = defaultState[3];
			displayLinkSpeeds = defaultState[4];
		}
		
		/**
		 * Constructor with all data initialized
		 * @param waypoints if waypoints should be displayed
		 * @param destinations if destinations should be displayed
		 * @param destinationLabels if destination labels should be displayed
		 * @param links if links should be displayed
		 * @param linkSpeeds if link speed labels should be displayed
		 */
		public DisplayState(boolean waypoints, boolean destinations, boolean destinationLabels, boolean links, boolean linkSpeeds){
			displayWaypoints = waypoints;
			displayDestinations = destinations;
			displayDestinationLabels = destinationLabels;
			displayLinks = links;
			displayLinkSpeeds = linkSpeeds;
		}
		
		@Override
		public String toString(){
			return "Waypoints: "+displayWaypoints+" Destinations: "+displayDestinations+" Destination Labels: "
			+displayDestinationLabels+" Links: "+displayLinks+" Link Speeds: "+displayLinkSpeeds;
		}
	}
	
	/**
	 * Interface for a class that wants to be notified when the display state changes
	 * @author Sam Crow
	 *
	 */
	public interface DisplayChangeListener {
		/**
		 * Called whenever the user changes the display state
		 * @param s The new DisplayState
		 */
		public void displayStateChanged(DisplayState s);
	}


	@Override
	public void itemStateChanged(ItemEvent e) {
		Object box = e.getSource();
		if(box.equals(displayWaypointsBox)){
			if(e.getStateChange() == ItemEvent.DESELECTED){
				state.displayWaypoints = false;
			}else{
				state.displayWaypoints = true;
			}
		}
		if(box.equals(displayDestinationsBox)){
			if(e.getStateChange() == ItemEvent.DESELECTED){
				state.displayDestinations = false;
			}else{
				state.displayDestinations = true;
			}
		}
		if(box.equals(displayDestinationLabelsBox)){
			if(e.getStateChange() == ItemEvent.DESELECTED){
				state.displayDestinationLabels = false;
			}else{
				state.displayDestinationLabels = true;
			}
		}
		if(box.equals(displayLinksBox)){
			if(e.getStateChange() == ItemEvent.DESELECTED){
				state.displayLinks = false;
			}else{
				state.displayLinks = true;
			}
		}
		if(box.equals(displayLinkSpeedsBox)){
			if(e.getStateChange() == ItemEvent.DESELECTED){
				state.displayLinkSpeeds = false;
			}else{
				state.displayLinkSpeeds = true;
			}
		}
		
		//tell the listener, if there is one
		if(listener != null){
			listener.displayStateChanged(state);
		}
	}
	
}
