package org.samcrow;
import java.util.Collection;
import java.util.HashSet;

import org.lekan.graphics.GraphicsProgram;
import org.lekan.graphics.SGFrame;
import org.lekan.graphics.SGObject;
import org.lekan.graphics.SGObject.Origin;
import org.lekan.graphics.SGText;
import org.samcrow.DisplayOptions.DisplayState;

/**
 * Manages SGObjects to draw.
 * Background objects are drawn in the background every time.
 * Persistent objects are drawn after the background every time.
 * Transient objects are drawn after the persistent objects only once. Calling <code>redraw</code> subsequently will erase them.
 * @author Sam Crow
 *
 */
public class DrawQueue {
	private Collection<SGObject> transientObjects;
	private Collection<SGObject> persistentObjects;//SGObjects that are redrawn every time, after the background
	private Collection<SGObject> backgroundObjects;//SGObjects in the background that are redrawn every time
	private GraphicsProgram program;
	
	/**
	 * Constructor
	 * @param inProgram the GraphicsProgram that contains the SGFrame to draw to
	 */
	public DrawQueue(GraphicsProgram inProgram){
		transientObjects = new HashSet<SGObject>();
		backgroundObjects = new HashSet<SGObject>();
		persistentObjects = new HashSet<SGObject>();
		program = inProgram;
	}
	
	/**
	 * Constructor, specifying a collection of SGObjects to draw in the background every time
	 * @param inProgram the GraphicsProgram that contains the SGFrame to draw to
	 * @param inBackgroundObjects a Collection of background SGObjects
	 */
	public DrawQueue(GraphicsProgram inProgram, Collection<SGObject> inBackgroundObjects){
		transientObjects = new HashSet<SGObject>();
		backgroundObjects = new HashSet<SGObject>(inBackgroundObjects);
		persistentObjects = new HashSet<SGObject>();
		program = inProgram;
	}
	
	/**
	 * Add an object to the background list
	 * @param object the SGObject to add
	 */
	public void addBackground(SGObject object){
		backgroundObjects.add(object);
	}
	
	/**
	 * Add a Collection of objects to the background list
	 * @param objects the Collection of SGObjects to add
	 */
	public void addBackground(Collection<SGObject> objects){
		backgroundObjects.addAll(objects);
	}
	
	/**
	 * Add an object to the persistent list
	 * @param object the SGObject to add
	 */
	public void addPersistent(SGObject object){
		persistentObjects.add(object);
	}
	
	/**
	 * Add a Collection of objects to the persistent list
	 * @param objects the Collection of SGObjects to add
	 */
	public void addPersistent(Collection<SGObject> objects){
		persistentObjects.addAll(objects);
	}
	
	/**
	 * Add a feature that will be drawn when <code>redraw</code> is called, but that will be erased
	 * when <code>redraw</code> is called subsequent times.
	 * @param object The SGObject that will be drawn
	 */
	public void addTransient(SGObject object){
		transientObjects.add(object);
	}
	
	/**
	 * Add a Collection of objects to the transient list
	 * @param objects the Collection of SGObjects to add
	 */
	public void addTransient(Collection<SGObject> objects){
		transientObjects.addAll(objects);
	}
	
	/**
	 * Remove all SGObejcts from the array of persistent objects to draw
	 */
	public void clearPersistent(){
		persistentObjects.clear();
	}
	
	/**
	 * Clear the frame and draw all features added since the last call to <code>redraw</code>
	 * @param mode the mode to indicate
	 * @param state the current DisplayState
	 */
	public void redraw(String mode, DisplayState state){
		
		SGFrame frame = program.getFrame();
		frame.clearGraphics();
		for(SGObject object : backgroundObjects){
			if(object.origin != null){//origin specified
				if(shouldDraw(object.origin, state)){//if the mode indicates that this should be drawn
					frame.addObject(object);//draw it
				}
			}else{
				frame.addObject(object);//no origin specified, so add it
			}
		}
		frame.addObject(new SGText(mode, 8, 600));//draw mode text
		for(SGObject object : persistentObjects){
			if(object.origin != null){//origin specified
				if(shouldDraw(object.origin, state)){//if the mode indicates that this should be drawn
					frame.addObject(object);//draw it
				}
			}else{
				frame.addObject(object);//no origin specified, so add it
			}
		}
		for(SGObject object : transientObjects){
			if(object.origin != null){//origin specified
				if(shouldDraw(object.origin, state)){//if the mode indicates that this should be drawn
					frame.addObject(object);//draw it
				}
			}else{
				frame.addObject(object);//no origin specified, so add it
			}
		}
		transientObjects.clear();
	}
	
	private boolean shouldDraw(Origin origin, DisplayState state){
		if(origin == Origin.WAYPOINT){
			return state.displayWaypoints;
		}else if(origin == Origin.DESTINATION){
			return state.displayDestinations;
		}else if(origin == Origin.DESTINATION_LABEL){
			return state.displayDestinationLabels;
		}else if(origin == Origin.LINK){
			return state.displayLinks;
		}else if(origin == Origin.LINK_SPEED){
			return state.displayLinkSpeeds;
		}else{
			return false;
		}
	}
}

/*
 * queue.clearPersistent();//remove all things from the persistent (non-background) draw queue
		//add the objects in memory back to the draw queue
		if(s.displayDestinations || s.displayWaypoints || s.displayDestinationLabels){
			for(Node node : nodes){
				if(s.displayWaypoints && node instanceof Waypoint){
					queue.addPersistent(node.getDrawObject());
				}
				if(node instanceof Destination){
					if(s.displayDestinations){
						queue.addPersistent(node.getDrawObject());
					}
					if(s.displayDestinationLabels){
						queue.addPersistent(((Destination) node).getLabel());
					}
				}
				
			}
		}
		if(s.displayLinks || s.displayLinkSpeeds){
			for(Link link : links){
				if(s.displayLinks){
					queue.addPersistent(link.getDrawObject());
				}
				if(s.displayLinkSpeeds){
					queue.addPersistent(link.getLabel());
				}
			}
		}
		
		queue.redraw(mode);
		*/
