package org.samcrow;
import org.lekan.graphics.SGLine;
import org.lekan.graphics.SGObject;
import org.lekan.graphics.SGObject.Origin;
import org.lekan.graphics.SGText;


public class Link {
	public static final double DEFAULT_SPEED = 8;
	
	private Node start;
	private Node end;
	private double distance;//distance between end nodes in pixels
	private double speed;//speed in pixels per second at which this link can be traversed
	
	public Link(Node node1, Node node2, double inSpeed){
		start = node1;
		end = node2;
		speed = inSpeed;
		//calculate the distance
		double xDiff = Math.abs(node1.x - node2.x);
		double yDiff = Math.abs(node1.y - node2.y);
		distance = Math.hypot(xDiff, yDiff);
	}
	
	/**
	 * Get the speed of this link
	 * @return the speed
	 */
	public double speed(){
		return speed;
	}
	
	/**
	 * Get the time in seconds taken to traverse this link
	 * @return the time in seconds
	 */
	public double time(){
		return distance / speed;
	}
	
	/**
	 * Get the distance of this link
	 * @return the number of pixels in the length of this link
	 */
	public double distance(){
		return distance;
	}
	
	/**
	 * Get the start node of this link
	 * @return the start node
	 */
	public Node startNode(){
		return start;
	}
	
	/**
	 * Get the start node of this link
	 * @return the start node
	 */
	public Node endNode(){
		return end;
	}
	
	/**
	 * Get the SGObject that should be drawn to represent this link
	 * @return the SGObject, currently an SGLine, to be drawn
	 */
	public SGObject getDrawObject(){
		SGObject object = new SGLine(start.x, start.y, end.x, end.y);
		object.origin = Origin.LINK;
		return object;
	}
	
	/**
	 * Get the SGText SGObject to display the label for this Link
	 * @return the SGText object
	 */
	public SGText getLabel() {
		//return the text of the speed, with its top left corner in the middle of the line
		SGText text = new SGText(String.valueOf(speed), (start.x + end.x) / 2, (start.y + end.y) / 2);
		text.origin = Origin.LINK_SPEED;
		return text;
	}
	
	public String toString(){
		return "A Link between ("+start.x+", "+start.y+") and ("+end.x+", "+end.x+") with a length of "+distance+" pixels and speed of "+speed+" pixels per second";
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(distance);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + ((end == null) ? 0 : end.hashCode());
		temp = Double.doubleToLongBits(speed);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + ((start == null) ? 0 : start.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Link)) {
			return false;
		}
		Link other = (Link) obj;
		if (Double.doubleToLongBits(distance) != Double
				.doubleToLongBits(other.distance)) {
			return false;
		}
		if (end == null) {
			if (other.end != null) {
				return false;
			}
		} else if (!end.equals(other.end)) {
			return false;
		}
		if (Double.doubleToLongBits(speed) != Double
				.doubleToLongBits(other.speed)) {
			return false;
		}
		if (start == null) {
			if (other.start != null) {
				return false;
			}
		} else if (!start.equals(other.start)) {
			return false;
		}
		return true;
	}
}
