package org.samcrow;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.HashSet;
import org.lekan.graphics.SGEllipse;
import org.lekan.graphics.SGObject;

/**
 * Base class for nodes
 * @author Sam Crow
 *
 */
public abstract class Node implements Comparable<Node>{
	protected static final int CIRCLE_RADIUS = 4;
	
	protected int x;
	protected int y;
	
	protected Collection<Link> links;
	
	
	/**
	 * Construct a node with a given location and no links
	 * @param inX The X coordinate of the node
	 * @param inY The Y coordinate of the node
	 */
	public Node(int inX, int inY){
		x = inX;
		y = inY;
		links = new HashSet<Link>();
	}
	
	
	public final void addLink(Link inLink){
		links.add(inLink);
	}
	
	/**
	 * Get the SGObjects to draw this node
	 * @return a collection of SGObjects to draw
	 */
	public SGObject getDrawObject() {
		SGEllipse ellipse = new SGEllipse(x, y, CIRCLE_RADIUS, CIRCLE_RADIUS);
		return ellipse;
	}
	
	public final Collection<Link> getLinks(){
		return links;
	}
	
	/**
	 * Get the X-coordinate of this node
	 * @return the X-coordinate
	 */
	public final int getX(){
		return x;
	}
	
	/**
	 * Get the Y-coordinate of this node
	 * @return the Y-coordinate
	 */
	public final int getY(){
		return y;
	}
	
	/**
	 * Check if the location of a given MouseEvent is inside the circle that represents this Node
	 * @param e the MouseEvent to check
	 * @return if the MouseEvent was inside this Node's circle
	 */
	public boolean isInArea(MouseEvent e){
		double xDiff = Math.abs(e.getX() - x);
		double yDiff = Math.abs(e.getY() - y);
		double distance = Math.hypot(xDiff, yDiff);
		return distance < CIRCLE_RADIUS;
	}
	
	public String toString(){
		return "A Waypoint at the coordinates ("+x+", "+y+")";// with links "+links;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + x;
		result = prime * result + y;
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Node)) {
			return false;
		}
		Node other = (Node) obj;
		if (x != other.x) {
			return false;
		}
		if (y != other.y) {
			return false;
		}
		if(!links.equals(other.getLinks())){
			return false;
		}
		return true;
	}
	
	@Override
	public int compareTo(Node o){
		if(equals(o)){
			return 0;
		}
		if(x + y > o.x + o.y){
			return 1;
		}else if(x + y < o.x + o.y){
			return -1;
		}else{
			return 0;
		}
	}
}
