package org.samcrow;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.text.ParseException;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import org.lekan.graphics.GraphicsProgram;
import org.lekan.graphics.SGEllipse;
import org.lekan.graphics.SGImage;
import org.lekan.graphics.SGLine;
import org.samcrow.DisplayOptions.DisplayChangeListener;
import org.samcrow.DisplayOptions.DisplayState;
import org.samcrow.io.BasicIO;
import org.samcrow.io.IOResponse;

/**
 * Who needs Google Maps when you have Pathfinder?
 * 
 * @author Sam Crow
 *
 */
public class Pathfinder extends GraphicsProgram implements ActionListener, DisplayChangeListener {
	public static void main(String[] args) {
		new Pathfinder();
	}

	// Constants
	private static final int CANVAS_WIDTH = 1000;
	private static final int CANVAS_HEIGHT = 618;

	private static final String MAP_FILE_NAME = "Stanfordmap-1000x618.png";

	private static final String ADD_WAYPOINT = "Add Waypoint";
	private static final String ADD_DESTINATION = "Add Destination";
	
	private static final String MENU_SAVE_AS = "Save As...";
	private static final String MENU_OPEN = "Open...";
	
	private static final String MENU_CLEAR = "Clear";

	private static final String ROUTE_MODE = "Route Mode";

	//instance variables
	private String mode = ROUTE_MODE;

	private DrawQueue queue;

	private Collection<Node> nodes;
	private Collection<Link> links;
	
	private JFileChooser chooser;
	private boolean isOpening;

	private DisplayState state;

	private Node dragStartNode;

	private boolean isDragging;

	@Override
	public void setup() {
		mode = ROUTE_MODE;
		// Setup code goes here
		nodes = new HashSet<Node>();
		links = new HashSet<Link>();
		this.getFrame().setTitle("Pathfinder BETA!");
		this.getFrame().setDimensions(CANVAS_WIDTH, CANVAS_HEIGHT);
		this.addButton(ADD_WAYPOINT, SOUTH);
		this.addButton(ADD_DESTINATION, SOUTH);
		this.addButton(ROUTE_MODE, SOUTH);
		queue = new DrawQueue(this);
		
		//menu bar
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		menuBar.add(fileMenu);
		JMenuItem openMenuItem = new JMenuItem(MENU_OPEN);
		openMenuItem.addActionListener(this);
		fileMenu.add(openMenuItem);
		JMenuItem saveMenuItem = new JMenuItem(MENU_SAVE_AS);
		saveMenuItem.addActionListener(this);
		fileMenu.add(saveMenuItem);
		JMenu editMenu = new JMenu("Edit");
		JMenuItem clearMenuItem = new JMenuItem(MENU_CLEAR);
		editMenu.add(clearMenuItem);
		clearMenuItem.addActionListener(this);
		menuBar.add(editMenu);
		getFrame().getJFrame().setJMenuBar(menuBar);
		
		//create the display options, add them to the UI, set this as the DisplayChangeListener
		DisplayOptions options = new DisplayOptions(this, EAST);
		state = options.new DisplayState();
		
		queue.addBackground(new SGImage(0, 0, MAP_FILE_NAME));
		
		//file chooser setup
		chooser = new JFileChooser();
		chooser.addActionListener(this);

		//queue.addBackground(new SGImage(0, 0, MAP_FILE_NAME));
		queue.redraw(mode, state);
	}

	@Override
	public void run() {
		// Run code goes here
	}

	@Override
	public void mouseDragged(MouseEvent e){
		isDragging = true;
		if(mode.equals(ADD_WAYPOINT) || mode.equals(ADD_DESTINATION)){
			if(dragStartNode != null){//the mouse could be dragged when a line is not currently being drawn. Avoid a NullPointerException by accessing dragStartNode only if it is set.
				SGLine line = new SGLine(dragStartNode.x, dragStartNode.y, e.getX(), e.getY());
				queue.addTransient(line);
				queue.redraw(mode, state);
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//check if a waypoint was clicked
		dragStartNode = nodeOver(e);
	}

	@Override
	public void mouseReleased(MouseEvent e){
		if(mode.equals(ADD_WAYPOINT) || mode.equals(ADD_DESTINATION)){
			if(dragStartNode == null && !isDragging){//if dragStartNode it is null; meaning that dragging is not currently taking place
				//add a waypoint
				Node node = null;
				if(mode.equals(ADD_WAYPOINT)){
					node = new Waypoint(e.getX(), e.getY());
				}else if(mode.equals(ADD_DESTINATION)){
					String name = JOptionPane.showInputDialog(
							getFrame().getJFrame(),
							"Please enter a name for this destination",
							"Destination Name",
							JOptionPane.QUESTION_MESSAGE
					);
					if(name != null){//valid name entered
						node = new Destination(e.getX(), e.getY(), name);
					}else{
						return;
					}
				}
				nodes.add(node);
				queue.addPersistent(node.getDrawObject());
				if(node instanceof Destination){
					//add the name if applicable
					queue.addPersistent(((Destination) node).getLabel());
				}
				queue.redraw(mode, state);
			}else if(isDragging && dragStartNode != null){//drag in progress
				//check that the mouse has been released over a node
				Node nodeOver = nodeOver(e);
				if(nodeOver != null && !nodeOver.equals(dragStartNode)){//ensure that the drag did not start and end on the same object
					//drag is valid. Create a link.
					Link link = null;
					if(e.getModifiers() == 17){//if the shift key was held down during this drag
						String speedStr = JOptionPane.showInputDialog(
								getFrame().getJFrame(),
								"Please enter the speed in pixels per second at which this link can be traversed",
								"Link Speed",
								JOptionPane.QUESTION_MESSAGE
						);
						if(speedStr != null){//valid speed was entered
							double speed = Double.parseDouble(speedStr);
							link = new Link(dragStartNode, nodeOver, speed);//use the speed entered by the user
						}
					}else{//shift key not held down. Use the default speed.
						link = new Link(dragStartNode, nodeOver, Link.DEFAULT_SPEED);
					}
					//nodeOver and dragStartNode are valid references to Nodes that are already in the nodes instance variable.
					//Hooray for references.
					nodeOver.addLink(link);
					dragStartNode.addLink(link);
					links.add(link);
					queue.addPersistent(link.getDrawObject());
					queue.addPersistent(link.getLabel());
					queue.redraw(mode, state);

				}else{
					//invalid link
					queue.redraw(mode, state);//redraw to delete this invalid link
				}

				dragStartNode = null;//signify that dragging is not happening
			}
		}
		
		Node nodeOver = nodeOver(e);
		
		if(mode.equals(ROUTE_MODE)
				&& dragStartNode != null
				&& nodeOver != null
				&& dragStartNode instanceof Destination
				&& nodeOver instanceof Destination){
			//find route
			DijkstraAlgorithm finder = new DijkstraAlgorithm(new Graph(nodes, links));
			finder.execute(dragStartNode);
			LinkedList<Node> nodes = finder.getPath(nodeOver);
			for(Node node : nodes){
				SGEllipse ellipse = new SGEllipse(node.x, node.y, 6, 6);
				queue.addTransient(ellipse);
				System.out.println(node);
			}
			queue.redraw(mode, state);
		}
		
		isDragging = false;
	}

	@Override
	public void actionPerformed(ActionEvent e){
		String command = e.getActionCommand();
		
		//buttons
		if(command.equals(ADD_WAYPOINT)){
			mode = ADD_WAYPOINT;
			queue.redraw(mode, state);
		}
		if(command.equals(ADD_DESTINATION)){
			mode = ADD_DESTINATION;
			queue.redraw(mode, state);
		}
		if(command.equals(ROUTE_MODE)){
			mode = ROUTE_MODE;
			queue.redraw(mode, state);
		}
		
		//menu items
		if(command.equals(MENU_OPEN)){
			//open the given file; add its contents to the data in memory
			System.out.println("Opening...");
			isOpening = true;
			chooser.showOpenDialog(getFrame().getJFrame());
		}
		if(command.equals(MENU_SAVE_AS)){
			//save the current map to a file
			System.out.println("Saving...");
			chooser.showSaveDialog(getFrame().getJFrame());
		}
		if(command.equals(MENU_CLEAR)){
			nodes.clear();
			links.clear();
			queue.clearPersistent();
			queue.redraw(mode, state);
		}
		
		//file chooser items
		if(command.equals(JFileChooser.APPROVE_SELECTION)){
			//file choice confirmed
			System.out.println("File Confirmed.");
			File selectedFile = chooser.getSelectedFile();
			BasicIO io = new BasicIO(selectedFile);
			if(isOpening){
				//read file
				System.out.println("Opening file at "+selectedFile.getPath());
				IOResponse response = null;
				try {
					response = io.read();
				} catch (ParseException e1) {
					JOptionPane.showMessageDialog(getFrame().getJFrame(), e1.toString() + " at line " + e1.getErrorOffset(), "File Read Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
				if(response != null){
					nodes.addAll(response.nodes);
					links.addAll(response.links);
				}
				for(Node node : response.nodes){
					queue.addPersistent(node.getDrawObject());
					if(node instanceof Destination){
						queue.addPersistent(((Destination) node).getLabel());
					}
				}
				for(Link link : response.links){
					queue.addPersistent(link.getDrawObject());
					queue.addPersistent(link.getLabel());
				}
				queue.redraw(mode, state);
				isOpening = false;
			}else{//saving
				System.out.println("Saving to the file at "+selectedFile.getPath());
				io.write(nodes, links);
			}
		}
	}

	@Override
	public void displayStateChanged(DisplayState s) {
		state = s;
		queue.redraw(mode, state);
	}
	
	/**
	 * Check if a given MouseEvent is over the circle representing any of the active nodes
	 * @param e the MouseEvent to check
	 * @return the Node that the mouse event was over, null if it is not over any Node
	 */
	private Node nodeOver(MouseEvent e){
		for(Node node : nodes){
			if(node.isInArea(e)){
				return node;
			}
		}
		return null;
	}


}
