
package org.samcrow.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;

import javax.swing.JOptionPane;

import org.samcrow.Destination;
import org.samcrow.Link;
import org.samcrow.Node;
import org.samcrow.Waypoint;

/**
 * Handles IO for a basic file with a non-standard format
 * <br /><br />
 * The format is as follows:<br />
 * <code>
 * Node &lt;hash&gt; &lt;x&gt; &lt;y&gt; "&lt;name&gt";
 * </code>
 * <br />
 * if <code>name</code> is nonexistent; the node is considered a waypoint instead of a destination
 * <code>
 * <br />
 * Link &lt;start_node_hash&gt; &lt;end_node_hash&gt; &lt;speed&gt;<br />
 * #comment
 * </code>
 * <br />
 * Any Link must appear in the file after both of the Nodes that it links.
 * @author Sam Crow
 *
 */
public class BasicIO extends IOBase {
	
	/**
	 * The word at the start of a line that indicates that the line contains node information
	 */
	private static final String NODE = "Node";
	
	/**
	 * The word at the start of a line that indicates that the line contains link information
	 */
	private static final String LINK = "Link";
	
	/**
	 * The expected file version
	 */
	//private static final int VERSION = 2;
	

	/**
	 * Construct this with a given filename
	 * @param inFilename
	 */
	public BasicIO(String inFilename) {
		super(inFilename);
	}
	
	/**
	 * Constructor with a given File object
	 * @param inFile the File object
	 */
	public BasicIO(File inFile){
		super(inFile);
	}

	/* (non-Javadoc)
	 * @see org.samcrow.io.MapIO#write(java.util.Collection, java.util.Collection)
	 */
	@Override
	public void write(Collection<Node> nodes, Collection<Link> links) {
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(file));//open the file
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//header
		try {
			writer.write("# Pathfinder map file\n");
			Calendar cal = Calendar.getInstance();
		    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			writer.write("# Created at "+ sdf.format(cal.getTime())+"\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		//write nodes
		for(Node node : nodes){
			String name = "";
			if(node instanceof Destination){//if this node is a destination, and therefore has a name
				name = " \"" + ((Destination)node).getName() + "\"";//name surrounded by double quotes
			}
			String line = "Node " + node.hashCode() + " " + node.getX() + " " + node.getY() + name +  "\n";
			try {
				writer.write(line);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		//write links
		for(Link link : links){
			String line = "Link " + link.startNode().hashCode() + " " + link.endNode().hashCode() + " " + link.speed() + "\n";
			try {
				writer.write(line);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		//write end of file comment
		try {
			writer.write("# EOF\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		try {
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/* (non-Javadoc)
	 * @see org.samcrow.io.MapIO#read()
	 */
	@Override
	public IOResponse read() throws ParseException {
		IOResponse retval = new IOResponse();
		
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(file));
		} catch (IOException e) {
			throw new ParseException("File could not be opened.", 0);
		}
		
		//read each line
		String line;
		int lineNumber = 0;
		try {
			while((line = reader.readLine()) != null){
				lineNumber++;
				String[] segments = line.split("\\s+");//split the line into segments separated by spaces
				if(segments[0].equals(NODE)){
					//parse node
					int x, y;
					try{
						x = Integer.parseInt(segments[2]);
						y = Integer.parseInt(segments[3]);
					}catch(NumberFormatException e){
						JOptionPane.showMessageDialog(null, "Malformed X or Y location value in line \""+line+"\"\n"+file.getPath()+":"+lineNumber, "Parse Error", JOptionPane.ERROR_MESSAGE);
						continue;//next line
					}
					
					//handle the name. The name may contain spaces, but it is surrounded by double quotes
					String name = null;
					if(segments.length >= 5){
						String nameFirstPart = segments[4];
						char nameFirstChar = nameFirstPart.charAt(0);
						if(nameFirstChar == '"'){
							//look for the segment with the end of the name
							int i = 4;
							for( ; i < segments.length; i++){
								String namePart = segments[i];
								char partLastChar = namePart.charAt(namePart.length() - 1);//get the last character
								if(partLastChar == '"'){
									break;
								}
							}
							//now "<name>" is made up of segments[4] + segments[5] + ... + segments[i]
							if(i == 4){//no spaces in name. The whole name is in the fourth segment
								name = segments[i];
								name = name.replace("\"", "");//remove the quotes
							}else{//some spaces in the name
								name = segments[4];
								for(int j = 0; j < i; j++ ){
									name += segments[j];//combine the segments
								}
								name = name.replace("\"", "");//remove the quotes
							}
						}
					}
					
					Node node = null;
					if(name != null){//Name included -> is a Destination
						node = new Destination(x, y, name);
					}else{//waypoint
						node = new Waypoint(x, y);
					}
					
					retval.nodes.add(node);
				}
				else if(segments[0].equals(LINK)){
					//parse link
					int startHash = 0;
					int endHash = 0;
					double speed = 0;
					try{
						startHash = Integer.parseInt(segments[1]);
						endHash = Integer.parseInt(segments[2]);
						speed = Double.parseDouble(segments[3]);
					}catch(NumberFormatException e){
						JOptionPane.showMessageDialog(null, "Malformed start hash, end hash, or speed value in line \"" + line + "\"\n"+file.getPath()+":"+lineNumber, "Parse Error", JOptionPane.ERROR_MESSAGE);
						continue;//next line
					}
					
					Node startNode = null;
					Node endNode = null;
					
					//loop through the existing nodes
					//This is slow.
					//This, comparing the hashes, works because the
					//Node implementation of hashCode() returns the
					//same hash for a given Node in different instances of the program execution
					for(Node node : retval.nodes){
						int nodeHash = node.hashCode();
						if(nodeHash == startHash){
							startNode = node;
						}
						if(nodeHash == endHash){
							endNode = node;
						}
					}
					
					//check that the nodes were found
					if(startNode == null){
						JOptionPane.showMessageDialog(null, "Start Node with hash " + startHash + " not found. Referenced from \"" + line + "\"\n"+file.getPath()+":"+lineNumber, "Parse Error", JOptionPane.ERROR_MESSAGE);
						continue;//next line
					}if(endNode == null){
						JOptionPane.showMessageDialog(null, "End Node with hash " + startHash + " not found. Referenced from \"" + line + "\"\n"+file.getPath()+":"+lineNumber, "Parse Error", JOptionPane.ERROR_MESSAGE);
						continue;//next line
					}
					
					//the references for this link are good.
					//so add it
					Link link = new Link(startNode, endNode, speed);
					retval.links.add(link);
					startNode.addLink(link);
					endNode.addLink(link);
				}
			}
		} catch (IOException e) {
			throw new ParseException("File could not be read.", 0);
		}
		
		return retval;
	}

}
